//* dcanvas.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Jan 2004

#ifndef VGL_DCANVAS_H
#define VGL_DCANVAS_H

#include <vgl/dvgl/defines.h>
#include <vgl/canvas.h>
#include <utl/client.h>
#include <ds/pool.h>
#include <ds/queue.h>
#include <pthread.h>

class VGL_API VglDCanvas : public VglCanvas
{
public:
  typedef bool (*CanvasFunction)(VglDCanvas*,void*);
  typedef void (*ErrorFunction)(VglDCanvas*,void*);
  class Tile
  {
  public:
    int x;
    int y;
    int width;
    int height;
    int read_x;
    int read_y;
    int read_width;
    int read_height;
    int node;
    int primcount;
    double measureddt;
    double dt;
    DsArray<unsigned char> parameters;
    DsArray<unsigned char> *compbuffer;
    int level;
    bool userparamsavailable;
    bool ready;
    bool received;
    bool drawn;
    Tile () : compbuffer(0)
    {
    }
    virtual ~Tile ()
    {
    }
    virtual void Init ()
    {
      ready = false;
      received = false;
      drawn = false;
      compbuffer = 0;
    }
  };
  class Frame
  {
//    int m_id; 
  public:
    // tiles composing frame
    DsArray<Tile*> tiles;
    // number of tiles composing frame
    int numtiles;
    // frame number (0=first frame)
    int number;
    // number of tile parameters sent
    int numsent;
    // number of tile ready signals received
    int numsignals;
    // number of frames of latency
    int latency;
    // time when previous frame was issued (when this frame is completed, the latency of this last frame will be the completion time minus this time)
    double latencylastissue;
    // time when the frame was issued
    double issuetime;
    // time spent receiving this frame data
    double nettime;
    // total bytes received for frame
    unsigned int compbuffersize;
    // frame completely drawn flag
    bool completed;
    // frame tile load order: loadorder[0] = heaviest tile
    DsArray<int> loadorder;
/*
    bool GetUseBufferManager()
    {
      return m_id!= -1;
    }
    void SetID (int id)
    {
      m_id = id;
    }
    void SetNoBufferManager ()
    {
      SetID(-1);
    }
    int GetID ()
    {
      return m_id;
    }
*/
    void SetTile (int index, Tile *tile)
    {
      tiles[index] = tile;
    }
    Tile* GetTile (int index)
    {
      return tiles[index];
    }
    void Init (int number, int numtiles, double issuetime, int latency, double latencylastissue)
    {
      this->numsent = 0;
      this->numsignals = 0;
      this->completed = false;
      this->number = number;
      this->issuetime = issuetime;
      this->latency = latency;
      this->latencylastissue = latencylastissue;
      this->numtiles = numtiles;
      this->tiles.Resize(numtiles);
    }
  };
private:
  VglCanvas* m_canvas;
  DsPool<DsArray<unsigned char> > m_userparamspool;
  DsPool<DsArray<unsigned char> > m_compbufferpool;
  DsPool<Tile> m_tilepool;
  DsPool<Frame> m_framepool;
  DsQueue<Frame*> m_frames;
  pthread_mutex_t m_mutexparams;
  pthread_mutex_t m_mutextilepool;
  pthread_mutex_t m_mutexcompbufferpool;
  pthread_mutex_t m_mutexframepool;
  pthread_mutex_t m_mutexqueue;
  pthread_cond_t m_condqueue;
  pthread_mutex_t m_mutexready;
  pthread_cond_t m_condready;
  pthread_mutex_t m_mutexreceived;
  pthread_cond_t m_condreceived;
  pthread_mutex_t m_mutexnewcompletedframe;
  pthread_cond_t m_condnewcompletedframe; 
  // callbacks
  CanvasFunction m_init_func;
  void* m_init_data;
  VglReshapeFunction m_reshape_func;
  void* m_reshape_data;
  CanvasFunction m_prerender_func;
  void* m_prerender_data;
  VglFunction m_redraw_func;
  void* m_redraw_data;
  ErrorFunction m_error_func;
  void* m_error_data;
  typedef struct
  {
    char* ip;
    int portcontrol;
    int portbuffer;
  } ServerInfo;
  DsArray<ServerInfo> m_serverinfos;
  DsArray<Frame*> m_lastframes;
  int m_level;
  int m_mode;
  int m_bpp;
  int m_framehistorysize;
  int m_maxlatency;
  // flags
  int m_distmode;
  bool m_initializing;
  bool m_error;
  bool m_showdepthcomp;
  // user parameters
  DsQueue<DsArray<unsigned char>*> m_userparamsqueue;
  DsArray<unsigned char>* m_userparams;
protected:
  int m_averagenumframes;
  // frame parameters
  AlgMatrix m_modl;
  int m_width, m_height;
  float m_xmin, m_xmax, m_ymin, m_ymax, m_zmin, m_zmax;
  bool m_ortho;
  // flags
  int m_nframes;
  // arrays
  DsArray<UtlClient> m_servercontrol;
  DsArray<UtlClient> m_serverbuffer;
  DsArray<unsigned char> m_buffer;
public:
  void ControlThread ();
  void ReceiveThread ();
  virtual void Reshape (int w, int h);
private:
  void ControlThreadSimple ();
  void ControlThreadInterc ();
protected:
  static AlgMatrix CalcProjection (int totalwidth, int totalheight, float xmin, float xmax, float ymin, float ymax, float zmin, float zmax, bool ortho, int x, int y, int w, int h);
  virtual bool Connect ();
  bool InitServers ();
  bool PreAllocUserParams (int framesoflatency);
  bool PreAllocFrames (int numframes, int numtiles);
  bool PreAllocBuffers (int tilesperframe, int maxactiveframes);
  int GetBPP ()
  {
    return m_bpp;
  }
  // user parameter functions 
  void CreateUserParams ();
  void SetFrameParams ();
  void LockUserParams ();
  void UnlockUserParams ();
  DsArray<unsigned char>* GetUserParams ();
  void ReleaseUserParams (DsArray<unsigned char>* params);
  // buffer functions
  DsArray<unsigned char>* CreateBuffer ();
  void ReleaseBuffer (DsArray<unsigned char> *buffer);
  // tile functions
  Tile* CreateTile ();
  void ReleaseTile (Tile* tile);
  unsigned char* DecompressTile (Tile *tile, bool uniquememory=false);
  int GetTileBuffersNum (unsigned char **buffers);
  void GetTileBufferInfo (unsigned char **buffers, int *which, int *x, int *y, int *w, int *h);
  void DrawTileBuffers (unsigned char *buffer);
  unsigned char *GetTileBuffer (int i);
  // thread functions
  bool StartControlThread ();
  bool StartReceiveThread ();
  // frame functions
  Frame* CreateFrame ();
  void ReleaseBuffers (Frame* tile);
  void ReleaseFrame (Frame* tile);
  Frame* GetFrame (int framenumber);
  Frame* GetLastCompletedFrame ();
  bool WaitForNewTilesReady    (Frame *frame, int numtiles);
  bool WaitForNewTilesReceived (Frame *frame, int numtiles);
  void LockFrameQueue ();
  void UnlockFrameQueue ();
  void WaitForNewFramesOnQueue ();
  void SignalNewFrameOnQueue ();
  void LockReadyFlags ();
  void UnlockReadyFlags ();
  void SignalNewTileReady ();
  void LockReceivedFlags ();
  void UnlockReceivedFlags ();
  void SignalNewTileReceived ();
  bool WaitForNewCompletedFrame ();
  void SignalNewCompletedFrame ();
  void PackFrameTileParameters (
      DsArray<unsigned char>* dest,
      double frametime,
      DsArray<unsigned char>* userparams,
      DsArray<unsigned char>* techparams,
      const float *drawprojection,
      const float *cullprojection,
      const float *view,
      int readx,
      int ready,
      int readwidth,
      int readheight,
      int drawx,
      int drawy,
      int drawwidth,
      int drawheight,
      int complevel,
      int tileindex
    );
  void SmoothFrameTimes (Frame* frame);
  int GetMaxLatency ()
  {
    return m_maxlatency;
  }
  virtual Frame* CreateNextFrame (Frame *lastcompletedframe, Frame *lastsignaledframe, int framenumber, int numnodes) = 0;
  virtual void PackTiles (Frame* frame, DsArray<unsigned char>* userparams) = 0;
private:
  void PackNextFrame (Frame *currframe, int numnodes);
public:
  virtual Tile* AllocTile () = 0;
  virtual Frame* AllocFrame () = 0;
public:
  VglDCanvas (VglCanvas* canvas);
  virtual ~VglDCanvas ();

  VglCanvas* GetCanvas() const
  {
    return m_canvas;
  }
  enum Mode {
    ALPHA = 2,
  //  DEPTH = 16,
  };
  void SetMode (int mode);
  void SetDistMode (int mode)
  {
    m_distmode = mode;
  }
  int GetDistMode ()
  {
    return m_distmode;
  }
  void SetFrameHistorySize (int size)
  {
    m_framehistorysize = size;
  }
  int GetFrameHistorySize ()
  {
    return m_framehistorysize;
  }
  void SetAverageNumFrames (int numframes)
  {
    if (numframes > 0)
      m_averagenumframes = numframes;
  }
  void SetShowDepthComplexity (bool flag)
  {
    m_showdepthcomp = flag;
  }
  bool GetShowDepthComplexity ()
  {
    return m_showdepthcomp;
  }
  void SetInitFunc (CanvasFunction func, void* data=0)
  {
    m_init_func = func;
    m_init_data = data;
  }
  CanvasFunction GetInitFunc ()
  {
    return m_init_func;
  }
  void* GetInitData ()
  {
    return m_init_data;
  }
  void SetPreRenderFunc (CanvasFunction func, void* data=0)
  {
    m_prerender_func = func;
    m_prerender_data = data;
  }
  CanvasFunction GetPreRenderFunc ()
  {
    return m_prerender_func;
  }
  void* GetPreRenderData ()
  {
    return m_prerender_data;
  }
  void SetErrorFunc (ErrorFunction func, void* data=0)
  {
    m_error_func = func;
    m_error_data = data;
  }
  ErrorFunction GetErrorFunc ()
  {
    return m_error_func;
  }
  void* GetErrorData ()
  {
    return m_error_data;
  }
  void AddServer (char* ip, int portcontrol, int portbuffer);
  virtual int GetNumServers ()
  {
    return m_serverinfos.Size();
  }
  void ShutDown ();
  int GetLevel () const
  {
    return m_level;
  }
  void SetLevel (int l)
  {
    m_level = l;
  }
  bool SendBoolean (bool b);
  bool SendInt (int v);
  bool SendUInt (unsigned int u);
  bool SendUShort (unsigned short s);
  bool SendFloat (float f);
  bool SendDouble (double d);
  bool SendString (char* s);
  bool SendFloatArray (unsigned int n, const float* f);
  bool SendUCharArray (unsigned int n, const unsigned char* c);

  bool SendBoolean (int server, bool b);
  bool SendInt (int server, int v);
  bool SendUInt (int server, unsigned int u);
  bool SendUShort (int server, unsigned short s);
  bool SendFloat (int server, float f);
  bool SendDouble (int server, double d);
  bool SendString (int server, char* s);
  bool SendFloatArray (int server, unsigned int n, const float* f);
  bool SendUCharArray (int server, unsigned int n, const unsigned char* c);

  void SendBoolean (DsArray<unsigned char> *array, bool b);
  void SendInt (DsArray<unsigned char> *array, int v);
  void SendUInt (DsArray<unsigned char> *array, unsigned int u);
  void SendUShort (DsArray<unsigned char> *array, unsigned short s);
  void SendFloat (DsArray<unsigned char> *array, float f);
  void SendDouble (DsArray<unsigned char> *array, double d);
  void SendString (DsArray<unsigned char> *array, char* s);
  void SendFloatArray (DsArray<unsigned char> *array, unsigned int n, const float* f);
  void SendUCharArray (DsArray<unsigned char> *array, unsigned int n, const unsigned char* c);

  bool GetError ()
  {
    return m_error;
  }
  void SetError (bool flag);
  bool IsConnected ()
  {
    return !m_initializing;
  }
  int GetFramesOnQueue ()
  {
    return m_frames.Size();
  }
  Frame* GetFrameAt (int index)
  {
    Frame *f;
    m_frames.Get(index,&f);
    return f;
  }
  void EnqueueFrame (Frame *f);
  Frame* DequeueFrame ()
  {
    Frame *f;
    m_frames.Dequeue(&f);
    return f;
  }
  // pure virtual methods
  virtual bool Init () = 0;
  // proxies to the view surface
  virtual VglCamera* GetCamera ();
  virtual void SetCamera (VglCamera* camera);
  virtual void SwapBuffers ();
  virtual void PostRedraw ();
  virtual void SetRasterCursor (bool on);
  virtual void DrawItem (VglFunction func, void* data=0);
  virtual void CenterCursor();
  // proxies to the real canvas
  virtual void SetNotifier(VglNotifier* notifier);
  virtual VglNotifier* GetNotifier ();
  virtual bool AddHandler (VglHandler* handler);
  virtual bool RemHandler (VglHandler* handler);
  virtual DsPList<VglHandler>::Iterator GetHandlers ();
  virtual void ClearHandlers ();
  virtual bool AddIdle (VglFunction func, void* data, int repeat);
  virtual bool RemIdle (VglFunction func, void* data);
  virtual void SetRedrawFunc (VglFunction func, void* data=0);
  virtual void SetReshapeFunc (VglReshapeFunction func, void* data=0);
  virtual VglFunction GetRedrawFunc ();
  virtual void* GetRedrawData ();
  virtual VglReshapeFunction GetReshapeFunc ();
  virtual void* GetReshapeData ();
  virtual void SetKeyboardFunc (VglKeyboardFunction func, void *data=0); 
  virtual void SetMouseButtonFunc (VglMouseButtonFunction func, void *data=0); 
  virtual void SetMouseMotionFunc (VglMouseMotionFunction func, void *data=0);
  virtual void SetMouseEntryFunc (VglMouseEntryFunction func, void *data=0);
  virtual void SetAudienceDistance (float dist);
  virtual float GetAudienceDistance ();
  virtual void SetDimension (float width, float height);
  virtual void SetShift (float shiftx, float shifty);
  virtual void SetRotation (float angle, float x=0.0f, float y=1.0f, float z=0.0f);
  virtual void EnableMapping (float magfactor=1.0f);
  virtual void DisableMapping ();
  //virtual void SetCursor (VglCursor* cursor); //TODO
  virtual void Activate ();
  virtual bool IsActive ();
  virtual int GetWidth ();
  virtual int GetHeight ();
  virtual bool IsStereo ();
protected:
};

#endif

