//* dcanvas.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Jan 2004

#ifndef VGL_DCANVAS_H
#define VGL_DCANVAS_H

#include <vgl/canvas.h>
#include <utl/client.h>
#include <ds/list.h>
#include <ds/queue.h>
#include <ds/pool.h>

class VGL_API VglDCanvas : public VglCanvas
{
public:
	typedef bool (*Func)(VglDCanvas*,void*);
	class Tile
	{
	public:
		int x, y;
		int width, height;
    int balwidth, balheight;
		double dt;
    double avgdt;
		bool received;
    bool decompressed;
    bool drawn;
    int primcount;
    int level;
		DsArray<unsigned char> parameters;
		unsigned int parsize;
		DsArray<unsigned char> compbuffer;
		DsArray<unsigned char> buffer;
    unsigned char* drawbuffer;
    unsigned long compbuffersize;
    unsigned long buffersize;
	};
	class Frame
	{
	public:
		DsArray<Tile*> tiles;
    DsArray<int> h_edges;
    DsArray<int> v_edges;
    DsArray<int> h_baledges;
    DsArray<int> v_baledges;
		DsArray<bool> ready;
		DsArray<int> indices;
		DsArray<int> nodetiles;
		int number;
    int numsent;
    int latency;
    double time;
    double maxdt;
    double alpha;
		bool completed;
		bool balance_calculated;
		bool baledges_calculated;
	};
private:
	DsPool<Tile> m_tilepool;
	DsPool<Frame> m_framepool;
	DsPool<DsArray<unsigned char> > m_userparamspool;
	DsQueue<Frame*> m_frames;
	VglCanvas* m_canvas;
	// callbacks
	Func m_init_func;
	void* m_init_data;
	Func m_prerender_func;
	void* m_prerender_data;
	void (*m_error_func)(VglDCanvas*,void*);
	void* m_error_data;
	typedef struct
	{
		char* ip;
		int portcontrol;
		int portbuffer;
	} ServerInfo;
  typedef struct
  {
    double time;
    double dtsum;
    double maxdt;
    double loadimbal;
    double compltime;
    int latency;
    int primcount;
  } FrameInfo;
	DsArray<UtlClient> m_servercontrol;
	DsArray<UtlClient> m_serverbuffer;
	DsArray<ServerInfo> m_serverinfos;
	DsArray<unsigned char> m_buffer;
  int m_level;
	bool m_showlines;
	bool m_showfps;
	// flags
	bool m_initializing;
	bool m_error;
	// thread control
	pthread_mutex_t m_mutexqueue;
	pthread_mutex_t m_mutexready;
	pthread_mutex_t m_mutexreceived;
	pthread_mutex_t m_mutexdecompressed;
	pthread_mutex_t m_mutexparams;
  pthread_mutex_t m_mutexnewcompletedframe;
	pthread_cond_t m_condready;
	pthread_cond_t m_condqueue;
	pthread_cond_t m_condreceived;
	pthread_cond_t m_conddecompressed;
  pthread_cond_t m_condnewcompletedframe; 
	DsArray<int> m_whichframe;
	// fps control
	double m_fps, m_t0, m_net_t, m_avgnet, m_lat, m_avglat, m_decomp_t, m_avgdecomp, m_write_t, m_avgwrite;
  double m_avgimbal, m_imbal_acc;
  double m_last_compl_t;
  DsArray<FrameInfo> m_framehistory;
  double m_firstframetime;
	int m_nframes;
	int m_currframe;
	// user parameters
	DsQueue<DsArray<unsigned char>*> m_userparamsqueue;
	DsArray<unsigned char>* m_userparams;
	int m_userparamssize;
	// frame parameters
	AlgMatrix m_modl;
	int m_width, m_height;
	float m_xmin, m_xmax, m_ymin, m_ymax, m_zmin, m_zmax;
	bool m_ortho;
	
	static void RedrawCB (void* data);
	static void ReshapeCB (int w, int h, void* data);
	void Reshape (int w, int h);
	AlgMatrix CalcProjection (int totalwidth, int totalheight, float xmin, float xmax, float ymin, float ymax, float zmin, float zmax, bool ortho, int x, int y, int w, int h);
	Frame* GetFrame (int framenumber);
	void SendBoolean (Tile* tile, bool b);
	void SendInt (Tile* tile, int v);
	void SendUInt (Tile* tile, unsigned int u);
	void SendUShort (Tile* tile, unsigned short s);
	void SendFloat (Tile* tile, float f);
	void SendDouble (Tile* tile, double d);
	void SendString (Tile* tile, char* s);
	void SendFloatArray (Tile* tile, const float* f, unsigned int n);
	static void* ControlThread (void*);
	static void* ReceiveThread (void*);
	static void* DecompressThread (void*);
	void ControlThread ();
	void ReceiveThread ();
	void DecompressThread ();
	void CalcEdgePositions (Frame* frame, int numnodes, int whichedges);
  void CalcEdgeMovement (Frame* frame, Frame* lastframe, double alpha, int numnodes);
  void CalcTileDimensions (Frame* frame, int numnodes);
  void AdjustEdges (Frame* frame, int numnodes);
	void CalcFrameBalance (Frame* frame, int numnodes);
public:
  VglDCanvas (VglCanvas* canvas);
  virtual ~VglDCanvas ();

	VglCanvas* GetCanvas() const
	{
		return m_canvas;
	}
  void SetInitFunc (Func func, void* data=0)
	{
		m_init_func = func;
		m_init_data = data;
	}
  void SetPreRenderFunc (Func func, void* data=0)
	{
		m_prerender_func = func;
		m_prerender_data = data;
	}
  void SetErrorFunc (void (*func)(VglDCanvas*,void*), void* data=0)
	{
		m_error_func = func;
		m_error_data = data;
	}
	void AddServer (char* ip, int portcontrol, int portbuffer);
	bool Init ();
  void ShutDown ();
	void SetShowFPS (bool flag)
	{
		m_showfps = flag;
	}
	bool GetShowFPS ()
	{
		return m_showfps;
	}
	void SetShowLines (bool flag)
	{
		m_showlines = flag;
	}
	bool GetShowLines ()
	{
		return m_showlines;
	}
  int GetLevel ()
  {
    return m_level;
  }
  void SetLevel (int l)
  {
    m_level = l;
  }
	bool SendBoolean (bool b);
	bool SendInt (int v);
	bool SendUInt (unsigned int u);
	bool SendUShort (unsigned short s);
	bool SendFloat (float f);
	bool SendDouble (double d);
	bool SendString (char* s);
	bool SendFloatArray(const float* f, unsigned int n);

	bool GetError ()
	{
		return m_error;
	}
	double GetFPS ()
	{
		return m_fps;
	}
	double GetLatency ()
	{
		return m_avglat;
	}
	double GetNetTime ()
	{
		return m_avgnet;
	}
	double GetDecompressionTime ()
	{
		return m_avgdecomp;
	}
  double GetWriteTime ()
  {
    return m_avgwrite;
  }
  bool SaveFrameTimeHistory (char* filename);
	virtual void Redraw ();
  virtual void Activate ();
  virtual int GetWidth ();
  virtual int GetHeight ();
  virtual void SetRasterCursor (bool on);
  virtual bool IsStereo ();
  virtual void SwapBuffers ();
  virtual void PostRedraw ();
  virtual void DrawItem (VglFunction func, void* data=0);
  virtual void CenterCursor();
};

#endif

