//* dcanvasserver.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Jan 2004

#ifndef VGL_DCANVASSERVER_H
#define VGL_DCANVASSERVER_H

#include <vgl/dvgl/defines.h>
#include <vgl/cnv/pbuffercanvas.h>
#include <utl/pbuffer.h>
#include <utl/server.h>
#include <ds/list.h>
#include <ds/array.h>
#include <string.h>

class NetworkHandler;

class VGL_API VglDCanvasServer
{
public:
  typedef bool (*InitFunc)(VglDCanvasServer*,void* data);
  typedef void (*ReshapeFunc)(int w,int h,void* data);
  typedef bool (*PreRenderFunc)(VglDCanvasServer*,double frametime,bool paramsavailable,void* data);
  typedef int  (*RedrawFunc)(VglDCanvasServer*, void* data);
  typedef void (*FinishFunc)(VglDCanvasServer*, void* data);
private:
  NetworkHandler* m_net;
  VglPBufferCanvas m_pbuffer;
  int m_pbuffermode;
  UtlServer* m_serverparam;
  UtlServer* m_serverbuffer;
  UtlServer::Client m_clientparam;
  UtlServer::Client m_clientbuffer;
  DsArray<unsigned char> m_buffer;
  unsigned char *m_wrkmem;
  bool m_terminate;
  bool m_initializing;
  // stats
  int m_nframes;
  double m_idletimehistory[5];
  double m_idletime;
  // parameters
  unsigned char* m_parameters;
  unsigned int m_parsize;
  unsigned int m_paroffset;
  // culling parameters
  float m_cullproj[16];
  int m_reqx, m_reqy, m_reqwidth, m_reqheight;
  int m_drawx, m_drawy, m_drawwidth, m_drawheight;
  int m_tileindex;
  // reading parameters
  typedef struct
  {
    int x, y, w, h;
    int which;
  } ReadTile;
  DsArray<ReadTile> m_readtiles;
  // callbacks
  InitFunc m_init_func;
  void* m_init_data;
  FinishFunc m_finish_func;
  void* m_finish_data;
  PreRenderFunc m_prerender_func;
  void* m_prerender_data;
  RedrawFunc m_redraw_func;
  void* m_redraw_data;
  ReshapeFunc m_reshape_func;
  void* m_reshape_data;
  void ResetReadTileList ()
  {
    m_readtiles.Resize(0);
  }
  static void RedrawView (void* data);
  static void ReshapeView (int w, int h, void* data);
protected:
  void Redraw ();
  void Reshape (int w, int h);
public:
  VglDCanvasServer ();
  virtual ~VglDCanvasServer ();
  
  bool Init(int portparam, int portbuffer);

  void SetInitFunc (InitFunc func, void* data=0)
  {
    m_init_func = func;
    m_init_data = data;
  }
  void SetFinishFunc (FinishFunc func, void* data=0)
  {
    m_finish_func = func;
    m_finish_data = data;
  }
  void SetPreRenderFunc (PreRenderFunc func, void* data=0)
  {
    m_prerender_func = func;
    m_prerender_data = data;
  }
  void SetRedrawFunc (RedrawFunc func, void* data=0)
  {
    m_redraw_func = func;
    m_redraw_data = data;
  }
  void SetReshapeFunc (ReshapeFunc func, void* data=0)
  {
    m_reshape_func = func;
    m_reshape_data = data;
  }
  void SetPBufferMode (int mode);
  void MainLoop ();
  void Terminate ();
  
  void GetCullingWindow (int *reqx, int *reqy, int *reqwidth, int *reqheight)
  {
    *reqx = m_reqx;
    *reqy = m_reqy;
    *reqwidth = m_reqwidth;
    *reqheight = m_reqheight;
  }
  void GetDrawingWindow (int *drawx, int *drawy, int *drawwidth, int *drawheight)
  {
    *drawx = m_drawx;
    *drawy = m_drawy;
    *drawwidth = m_drawwidth;
    *drawheight = m_drawheight;
  }
  void GetCullingProjection (float cullproj[16])
  {
    memcpy(cullproj,m_cullproj,16*sizeof(float));
  }
  int GetTileIndex ()
  {
    return m_tileindex;
  }
  void InsertReadTile (int x, int y, int w, int h, int which)
  {
    int pos = m_readtiles.Size();
    m_readtiles[pos].x = x;
    m_readtiles[pos].y = y;
    m_readtiles[pos].w = w;
    m_readtiles[pos].h = h;
    m_readtiles[pos].which = which;
  }
  bool GetBoolean(bool* res);
  bool GetInt(int* res);
  bool GetUInt(unsigned int* res);
  bool GetUShort(unsigned short* res);
  bool GetFloat(float* res);
  bool GetDouble(double* res);
  bool GetString(char* dest, unsigned int size);
  bool GetFloatArray(unsigned int n, float* res);
  bool GetUCharArray(unsigned int n, unsigned char* res);

  int GetWidth ();
  int GetHeight ();
};

#endif

