//* dcanvasserver.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Jan 2004

#ifndef VGL_DCANVASSERVER_H
#define VGL_DCANVASSERVER_H

#include <vgl/cnv/pbuffercanvas.h>
#include <utl/server.h>
#include <ds/list.h>

class NetworkHandler;

class VGL_API VglDCanvasServer
{
public:
	typedef bool (*InitFunc)(VglDCanvasServer*,void* data);
	typedef void (*ReshapeFunc)(int w,int h,void* data);
	typedef bool (*PreRenderFunc)(VglDCanvasServer*,double frametime,bool paramsavailable,void* data);
  typedef int  (*RedrawFunc)(void* data);
private:
	NetworkHandler* m_net;
	VglPBufferCanvas m_pbuffer;
	UtlServer* m_serverparam;
	UtlServer* m_serverbuffer;
	UtlServer::Client m_clientparam;
	UtlServer::Client m_clientbuffer;
	DsArray<unsigned char> m_buffer;
	bool m_terminate;
	bool m_initializing;
	// stats
	int m_nframes;
	double m_idletimehistory[5];
	double m_idletime;
	// parameters
	unsigned char* m_parameters;
	unsigned int m_parsize;
	unsigned int m_paroffset;
	// callbacks
	InitFunc m_init_func;
	void* m_init_data;
	VglFunction m_finish_func;
	void* m_finish_data;
	PreRenderFunc m_prerender_func;
	void* m_prerender_data;
	RedrawFunc m_redraw_func;
	void* m_redraw_data;
	ReshapeFunc m_reshape_func;
	void* m_reshape_data;
	static void RedrawView (void* data);
	static void ReshapeView (int w, int h, void* data);
protected:
	void Redraw ();
	void Reshape (int w, int h);
public:
  VglDCanvasServer ();
  virtual ~VglDCanvasServer ();
	
	bool Init(int portparam, int portbuffer);

  void SetInitFunc (InitFunc func, void* data=0)
	{
		m_init_func = func;
		m_init_data = data;
	}
  void SetFinishFunc (VglFunction func, void* data=0)
	{
		m_finish_func = func;
		m_finish_data = data;
	}
	void SetPreRenderFunc (PreRenderFunc func, void* data=0)
	{
		m_prerender_func = func;
		m_prerender_data = data;
	}
  void SetRedrawFunc (RedrawFunc func, void* data=0)
	{
		m_redraw_func = func;
		m_redraw_data = data;
	}
	void SetReshapeFunc (ReshapeFunc func, void* data=0)
	{
		m_reshape_func = func;
		m_reshape_data = data;
	}
	void MainLoop ();
	void Terminate ();

	bool GetInt(int* res);
	bool GetBoolean(bool* res);
	bool GetUInt(unsigned int* res);
	bool GetUShort(unsigned short* res);
	bool GetFloat(float* res);
	bool GetFloatArray(float* res, unsigned int n);
	bool GetDouble(double* res);
	bool GetString(char* dest, unsigned int size);

  int GetWidth ();
  int GetHeight ();
};

#endif

