//* dcanvassortlast.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Oct 2004

#ifndef VGL_DCANVASSORTLAST_H
#define VGL_DCANVASSORTLAST_H

#include <vgl/dvgl/defines.h>
#include <ds/array.h>
#include <ds/queue.h>
#include <ds/pool.h>
#include <pthread.h>
#include "dcanvastiled.h"

//class VglBufferManager2;

class VGL_API VglDCanvasSortLast : public VglDCanvasTiled
{
public:
  class SortLastTile : public VglDCanvas::Tile
  {
    unsigned short planes;
  public:
    virtual void Init ()
    {
      planes = 0;
      Tile::Init();
    }
    void EnablePlane (int plane)
    {
      planes |= (1<<plane);
    }
    bool IsPlaneEnabled (int plane)
    {
      return planes & (1<<plane);
    }
    unsigned short GetPlanes ()
    {
      return planes;
    }
  };
  class SortLastFrame : public VglDCanvasTiled::TiledFrame
  {
  public:
    DsArray<int> loadorder;
    double alpha;
    SortLastTile* GetSLTile (int index)
    {
      return (SortLastTile*)tiles[index];      
    }
    void Init (int number, int numtiles, double issuetime, int latency, double latencylastissue)
    {
      TiledFrame::Init(number,numtiles,issuetime,latency,latencylastissue);
      alpha = 0.0; 
    }
  };
private:
//  VglBufferManager2 *m_bufman;
  typedef struct
  {
    int totalbuffersize;
    double time;
    double loadimbal;
    double rendertime;
    int latency;
    double latencytime;
    double nettime;
    int primcount;
  } FrameInfo;
  bool m_showlines;
  bool m_showfps;
  DsArray<unsigned char> m_techparams;
  // flags
  int m_distmode;
  int m_intsize;
  // thread control
  DsArray<int> m_whichframe;
  // fps control
  double m_fps, m_t0, m_net_t, m_avgnet, m_lat, m_avglat, m_avgbytes, m_decomp_t, m_avgdecomp, m_write_t, m_avgwrite, m_maxdt, m_avgmaxdt;
  unsigned long m_bytes;
  double m_avgimbal, m_imbal_acc;
  double m_last_compl_t;
  DsArray<FrameInfo> m_framehistory;
  DsArray<bool> m_enabled;
  double m_firstframetime;
  int m_currframe;
  double m_lastframeissue_t;
  bool m_sortlast;
//  bool m_usebuffer;
protected:
  void CalcFrustumPlanes (SortLastFrame *frame);
  void InitFrame (SortLastFrame *frame);
  virtual Frame* CalcFirstFrame (DsArray<unsigned char> *userparams);
  virtual Frame* CreateNextFrame (Frame *lastcompletedframe, Frame *lastsignaledframe, int framenumber, int numnodes);
  virtual void PackTiles (Frame* frame, DsArray<unsigned char>* userparams);
  // sets up the buffer manager
//  void SetupFrameBuffers (SortLastFrame* frame, int numtiles);
  // rendering thread
public:
  virtual Tile* AllocTile ()
  {
    return new SortLastTile();
  }
  virtual Frame* AllocFrame ()
  {
    return new SortLastFrame();
  }
public:
  VglDCanvasSortLast (VglCanvas* canvas);
  virtual ~VglDCanvasSortLast ();

  void SetDistMode (int mode)
  {
    m_distmode = mode;
  }
  int GetDistMode ()
  {
    return m_distmode;
  }
  void SetIntSize (int size)
  {
    m_intsize = size;
  }
  int GetIntSize ()
  {
    return m_intsize;
  }
  void SetShowFPS (bool flag)
  {
    m_showfps = flag;
  }
  bool GetShowFPS ()
  {
    return m_showfps;
  }
  void SetShowLines (bool flag)
  {
    m_showlines = flag;
  }
  bool GetShowLines ()
  {
    return m_showlines;
  }
  void SetEnabled (int i, bool f)
  {
    if (i < GetNumTiles())
      m_enabled[i] = f;
  }
  bool GetEnabled (int i)
  {
    if (i < GetNumTiles())
      return m_enabled[i];
    else
      return false;
  }
  void InvertEnabled ()
  {
    int i, n = GetNumTiles();
    for (i=0; i<n; i++)
      m_enabled[i] = !m_enabled[i];
  }
  void SetSortLast (bool f)
  {
    m_sortlast = f;
  }
  bool GetSortLast ()
  {
    return m_sortlast;
  }
  double GetFPS ()
  {
    return m_fps;
  }
  double GetLatency ()
  {
    return m_avglat;
  }
  double GetNetTime ()
  {
    return m_avgnet;
  }
  double GetAvgTransferredBytes ()
  {
    return m_avgbytes;
  }
  double GetDecompressionTime ()
  {
    return m_avgdecomp;
  }
  double GetAvgLoadImbalance ()
  {
    return m_avgimbal;
  }
  double GetAvgMaxDT ()
  {
    return m_avgmaxdt;
  }
  double GetWriteTime ()
  {
    return m_avgwrite;
  }
/*
  bool GetUseBuffer ()
  {
    return m_usebuffer;
  }
  void SetUseBuffer (bool f)
  {
    m_usebuffer = f;
  }
*/
  bool SaveFrameTimeHistory (char* localfilename, char* clusterfilename);
  // virtual methods
  virtual void Redraw ();
  virtual bool Init ();
};

#endif

