//* dcanvastiled.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Oct 2004

#ifndef VGL_DCANVASTILED_H
#define VGL_DCANVASTILED_H

#include <ds/array.h>
#include "dcanvas.h"

typedef struct 
{
  int nrows;
  int ncols[50];
} Division;

class VGL_API VglDCanvasTiled : public VglDCanvas
{
public:
  class TiledFrame : public VglDCanvas::Frame
  {
  public:
    double alpha;
    DsArray<int> h_edges;
    DsArray<int> v_edges;
    DsArray<int> h_baledges;
    DsArray<int> v_baledges;
    bool balance_calculated;
    bool baledges_calculated;
    typedef struct
    {
      int width, height;
    } BalTile;
    DsArray<BalTile> balance;
    Division useddivision;
    Division suggdivision;
    void Init (int number, int numtiles, double issuetime, int latency, double latencylastissue)
    {
      Frame::Init(number,numtiles,issuetime,latency,latencylastissue);
      balance_calculated = false;
      baledges_calculated = false;
      alpha = 0.0; 
    }
  };
  int m_numtiles;
  DsArray<int> m_presentrowtiles;
  DsArray<int> m_presentrowntiles;
  DsArray<int> m_presentrowcurr;
  DsArray<int> m_currpresentrowwidth;
  DsArray<int> m_rowtypes;
  DsArray<double> m_rowloads;
  DsArray<int> m_rowheights;
  bool m_linearbalance;
  bool m_balance;
protected:
  double m_lastframeissue_t;
  void CalcEdgePositions (TiledFrame* frame, int numtiles, int whichedges);
  void CalcEdgeMovement (TiledFrame* frame, TiledFrame* lastframe, double alpha, int numtiles);
  void CalcTileDimensions (TiledFrame* frame, int numtiles);
  void AdjustEdges (TiledFrame* frame, int numtiles);
  // calculates a new frame tile division based on the last frame
  void AdjustBalancedHeights (TiledFrame *lastframe, int numtiles, int expectedheight);
  void AdjustBalancedWidths (TiledFrame *lastframe, int numtiles, int expectedwidth);
  void CalcFrameBalance (TiledFrame* lastframe, int numtiles);
  // creates the next frame based on the previous frames
  virtual Frame* CalcFirstFrame (DsArray<unsigned char> *userparams);
  virtual Frame* CreateNextFrame (Frame *lastcompletedframe, Frame *lastsignaledframe, int framenumber, int numnodes);
  bool PreAlloc ();
  Division* GetDivision (int numtiles);
  // construtor
  VglDCanvasTiled (VglCanvas* canvas);
public:
  // destructor
  virtual ~VglDCanvasTiled ();
  // public methods
  bool SetNumTiles (int numtiles);
  int GetNumTiles ()
  {
    return m_numtiles;
  }
  void SetLinearBalance (bool flag)
  {
    m_linearbalance = flag;
  }
  bool GetLinearBalance ()
  {
    return m_linearbalance;
  }
  void SetBalance (bool flag)
  {
    m_balance = flag;
  }
  bool GetBalance ()
  {
    return m_balance;
  }
  // virtual methods
  virtual bool Init ();
};

#endif

