//* fixheightnavig.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Mar 2003

#ifndef VGL_FIX_HEIGHT_NAVIG_H
#define VGL_FIX_HEIGHT_NAVIG_H

#include "navig.h"
#include "selection.h"
#include <vgl/handler.h>
#include <alg/vector.h>

class VglViewSurface;

/**
 * This class implements a Navig Handler that keeps a constant distance to the ground.
 * That is done by tracing a ray from the camera position to the negative y axis 
 * direction and using this distance to correct the camera position height.
 */
class VGL_API VglFixHeightNavigHandler : public VglNavigHandler, public VglSelectionHandler
{
  float m_height;
  bool m_heightset;
  float m_zmin;
  float m_tolx, m_toly, m_range;

  float GetIntDistance(AlgVector position, AlgVector direction);
protected:
  virtual void DoNavigThrough ();
public:
  /**
   * Returns the event handler class classname.
   * \return Event handler class classname.
   */
  static char* CLASSNAME () { return "FixHeightNavigHandler"; }
  virtual char* ClassName () const { return CLASSNAME(); }
  /**
   * Constructs a new VglFixHeightNavigHandler.
   * \param surface The view surface associated with this handler.
   * \param maxvel  Maximum navigation velocity.
   * \param accel   Navigation acceleration.
   * \param tolx    Traced ray x tolerance in world coordinates.
   * \param toly    Traced ray y tolerance in world coordinates.
   * \param range   Traced ray range in world coordinates.
   */
  VglFixHeightNavigHandler (VglViewSurface* surface,
                            float maxvel=0.2,
                            float accel=1.0,
                            double tolx=0.1, 
                            double toly=0.1, 
                            float range=1000.0);
  /**
   * Destroys a VglFixHeightNavigHandler object.
   */
  virtual ~VglFixHeightNavigHandler ();
  virtual void ProcessFunction (VglSelectionHit* hits, int n);
  /**
   * Sets the new default height.
   * \param height Height in world coordinates.
   */
  void SetHeight (float height)
  {
    m_height = height;
    m_heightset = true;
  }
  void SetTolerance (float tolx, float toly)
  {
    m_tolx = tolx;
    m_toly = toly;
  }
  void SetRange (float range)
  {
    m_range = range;
  }
  void CorrectHeight ();
};

#endif
