//* modehandler.h
// fabraham@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Mar 2003

#ifndef VGL_MODE_HANDLER_H
#define VGL_MODE_HANDLER_H

#include <vgl/handler.h>
#include <vgl/defines.h>

class VglCanvas;
class VglSubCanvas;
struct Hnd_key;

/**
 * This class provides a mechanism to associate various event handlers to keys
 * and use these keys to switch between them.\n
 * This is done using the method AddHandler(handler,keycode).\n
 */
class VGL_API VglModeHandler : public VglHandler
{
  typedef DsPList<Hnd_key> HandlerList;
  VglCanvas *m_canvas;
  VglSubCanvas *m_subcanvas;
  VglHandler* m_curr;
  HandlerList m_hnd_list;
public:
  /**
   * Returns the event handler class classname.
   * \return Event handler class classname.
   */
  static char* CLASSNAME () { return "ModeHandler"; }
  virtual char* ClassName () const { return CLASSNAME(); }
  /**
   * Constructs a new VglModeHandler object,
   * which will be associated with canvas 'canvas'.
   */
  VglModeHandler (VglCanvas* canvas);
  /**
   * Constructs a new VglModeHandler object,
   * which will be associated with subcanvas 'subcanvas'.
   */
  VglModeHandler (VglSubCanvas* subcanvas);
  /**
   * Destroys a VglModeHandler object.
   */
  virtual ~VglModeHandler ();
  /**
   * Activates all handlers.
   */
  virtual void Begin ();
  /**
   * Deactivates all handlers.
   */
  virtual void End ();
  /**
   * Adds a new event handler to this switch.
   * \param hnd The new event handler.
   * \param switch_key Key that will switch to this event handler.
   * \return Success in the insertion.
   */
  bool AddHandler(VglHandler* hnd, int switch_key);
  /**
   * Changes the current handler to the one associated with key 'k'.
   * Returns the success in changing the handler.
   */
  bool ChangeHandler(int k);
  virtual int Keyboard (int k, int st, float x, float y);
};

#endif
