#ifndef VGL_NAVIG_H
#define VGL_NAVIG_H

#include <vgl/handler.h>
#include <alg/vector.h>
#include <alg/matrix.h>
#include <utl/clock.h>

class VglCanvas;
class VglViewSurface;

class VGL_API VglNavigHandler : public VglHandler
{
protected:
  bool m_center_cursor;
  bool m_cursor_centered;
  bool m_navigating;
  bool m_turning;
  bool m_onplane;
	bool m_drawcursor;
  char m_constraint;
  float m_maxvel, m_accel;
  double m_last_t, m_dt;
  bool m_key_w,m_key_s,m_key_a,m_key_d,m_key_r,m_key_f;
  AlgVector m_vel;
  AlgVector* m_rot_yaxis;
  UtlClock m_clock;
  VglViewSurface* m_surface;
  virtual void DoNavigThrough ();
  VglViewSurface* GetSurface ()
  {
    return m_surface;
  }
public:
  static char* CLASSNAME () { return "NavigHandler"; }
  virtual char* ClassName () const { return CLASSNAME(); }
  /**
   * Constructs a new navig handler.
   * 'surface' is the view surface associated with this handler.
   * 'maxvel' is the maximum navigation velocity. (world units/second)
   * 'accel' is the navigation acceleration. (world units/second^2)
   */
  VglNavigHandler (VglViewSurface* surface, float maxvel=0.2f, float accel=1.0f);
  virtual ~VglNavigHandler ();
  /**
   * Sets the maximum navigation velocity. (world units/second)
   */
  void SetMaxVelocity (float maxvel)
  {
    m_maxvel = maxvel;
  }
  /**
   * Sets the navigation acceleration. (world units/second^2)
   */
  void SetAcceleration (float accel)
  {
    m_accel = accel;
  }
  float GetMaxVelocity ()
  {
    return m_maxvel;
  }
  float GetAcceleration ()
  {
    return m_accel;
  }
  void SetOnPlane (bool flag)
  {
    m_onplane = flag;
  }
  bool GetOnPlane ()
  {
    return m_onplane;
  }
	/**
	 * Enables/disabled the drawing of a cursor in the direction of navigation.
	 */
	void SetDrawCursor (bool flag)
	{
		m_drawcursor = flag;
	}
  void SetCenterCursor (bool flag);
  bool GetCenterCursor ()
  {
    return m_center_cursor;
  }
  void SetYRotationAxis (AlgVector& axis);
  void SetYRotationAxis (float vx, float vy, float vz);
  void UnsetYRotationAxis ();
  AlgVector* GetYRotationAxis ()
  {
    return m_rot_yaxis;
  }
  virtual void Begin ();
  virtual void End ();
  virtual int Keyboard (int k, int st, float x, float y);
  virtual int MouseMotion (int bt, float x, float y);
  virtual int MouseEntry (bool st);
	virtual int PostRedraw ();
  // deprecated
  void SetAnimated (bool flag);
  bool GetAnimated ();
protected:
  static void NavigThrough (void* data);
  static void Redraw (void* data);
};

#endif
