/**
* Modulo de declaracao da classe VglNavigCollision. Esta classe sera responsavel por determinar se 
* a camera colidira ou nao ao navegar e tratara a resposta a colisao se ocorrer.
* @author Fabio Nakamura
*/

#ifndef VGL_NAVIG_COLLISION_H
#define VGL_NAVIG_COLLISION_H

#include "navig.h"

#define COLLISION_PRECISION 4

class COLLISION_NODE;
class VglCamera;

class VGL_API VglNavigCollision : public VglNavigHandler
{
	protected:

		/**
		* Este e o no de colisao da camera.
		*/
        COLLISION_NODE* Camera_Collision_Node;

		/**
		* Este e o no de colisao da hierarquia que sera testada contra 
		* a camera.
		*/
		COLLISION_NODE* Hierarchy;
    int Count;
    VglCamera *Camera;

		/**
		* Executa a navegacao e chama a deteccao de colisao
		*/
		virtual void DoNavigThrough(void);
	public:

		static char* CLASSNAME () { return "Navig_CollisionHandler"; }
		virtual char* ClassName () const { return CLASSNAME(); }

		/**
		* Constructs a new VglNavigCollision.
		* \param surface The view surface associated with this handler.
		* \param canvas  The canvas who owns this handler.
		* \param maxvel  Maximum navigation velocity.
		* \param accel   Navigation acceleration.
		*/
		VglNavigCollision (COLLISION_NODE* Camera_Node,COLLISION_NODE* Hierarchy
			,VglViewSurface* surface,float maxvel=0.2f, float accel=1.0f);
		virtual ~VglNavigCollision();


		/**
		* Metodos de Get / Set 
		*/

		COLLISION_NODE* Get_Camera_Node(void)
		{
			return(this->Camera_Collision_Node);
		}

		void Set_New_Camera_Node(COLLISION_NODE* New_Camera_Node)
		{
			this->Camera_Collision_Node = New_Camera_Node;
			return;
		}

		COLLISION_NODE* Get_Hierarchy(void)
		{
			return(this->Hierarchy);
		}

		void Set_New_Hierarchy(COLLISION_NODE* New_Hierarchy)
		{
			this->Hierarchy = New_Hierarchy;
			return;
		}

    int Validate ();

};

#endif

