#ifndef VGL_ROTATION_AXIS_H
#define VGL_ROTATION_AXIS_H

#include <alg/vector.h>
#include <vgl/handler.h>
#include <vgl/hnd/selection.h>
#include <vgl/hnd/pointpick.h>
#include <vgl/viewsurface.h>

#define VGL_ROTATION_AXIS_HANDLER "RotationAxisHandler"
#define VGL_ROTATION_HANDLER "RotationHandler"

class VGL_API VglRotationAxisHandler : public VglHandler
{
protected:
   bool m_moving;
   AlgVector m_pos;
   AlgVector m_axis;
   VglViewSurface *m_surface;

   AlgVector m_color;
   float m_thickness;
   float m_linesize;
   bool m_drawaxis;

   static void RepeatTransform (void* data);
   static void RenderAxis(void *data);

   void RenderAxis();

public:
   VglRotationAxisHandler(VglViewSurface* surface) :
      m_pos(0.0f,0.0f,0.0f),
      m_axis(0.0f,1.0f,0.0f),
      m_surface(surface),
      m_color(0.0f,0.0f,0.0f),
      m_thickness(1.0f),
      m_linesize(100.0f),
      m_drawaxis(true)
   {
   }

   virtual ~VglRotationAxisHandler(){}

   static char* CLASSNAME () { return VGL_ROTATION_AXIS_HANDLER; }
   virtual char* ClassName () const { return CLASSNAME(); }

   void SetAxis(const AlgVector &axis)
   {
      m_axis = axis;
   }

   void SetAxis(float x, float y, float z)
   {
      SetAxis(AlgVector(x, y, z));
   }

   void SetAxisPosition(const AlgVector &pos)
   {
      m_pos = pos;
   }

   void SetAxisPosition(float x, float y, float z)
   {
      SetAxisPosition(AlgVector(x,y,z));
   }

   AlgVector &GetAxis()
   {
      return m_axis;
   }

   void GetAxis(float *x, float *y, float *z)
   {
      *x = m_axis.x;
      *y = m_axis.y;
      *z = m_axis.z;
   }

   AlgVector &GetAxisPosition()
   {
      return m_pos;
   }

   void GetAxisPosition(float *x, float *y, float *z)
   {
      *x = m_pos.x;
      *y = m_pos.y;
      *z = m_pos.z;
   }

   void SetColor(const AlgVector color)
   {
      m_color = color;
   }

   void SetColor(float r, float g, float b)
   {
      SetColor(AlgVector(r,g,b));
   }

   AlgVector &GetColor()
   {
      return m_color;
   }

   void GetColor(float *r, float *g, float *b)
   {
      *r = m_color.x;
      *g = m_color.y;
      *b = m_color.z;
   }

   void SetThickness(float thickness)
   {
      m_thickness = thickness;
   }

   float GetThickness()
   {
      return m_thickness;
   }

   void SetLineSize(float size)
   {
      m_linesize = size;
   }

   float GetLineSize()
   {
      return m_linesize;
   }

   bool IsDrawingAxis()
   {
      return m_drawaxis;
   }

   void DrawAxis(bool draw)
   {
      m_drawaxis = draw;
   }

   virtual void End ();
   virtual int Keyboard (int k, int st, float x, float y);
   virtual int MouseButton (int bt, int st, float x, float y);
   virtual int MouseMotion (int bt, float x, float y);
};

class VGL_API VglRotationHandler : public VglRotationAxisHandler
{
public:
   VglRotationHandler(VglViewSurface* surface) :
      VglRotationAxisHandler(surface),
      m_pick(surface) {}
   virtual ~VglRotationHandler(){}

   static char* CLASSNAME () { return VGL_ROTATION_HANDLER; }
   virtual char* ClassName () const { return CLASSNAME(); }

   virtual int MouseButton (int bt, int st, float x, float y);

   void SetSelectionRenderFunc(VglSelectionHandler::SelectionFunc f, void *data)
   {
      m_pick.SetSelectionRenderFunc(f, data);
   }

protected:
   VglPickHandler m_pick;
   float m_x, m_y;
};


#endif