//* viewsurface.h
// celes@tecgraf.puc-rio.br
// Tecgraf/PUC-Rio
// Feb 2003

#ifndef VGL_VIEWSURFACE_H
#define VGL_VIEWSURFACE_H

#include "defines.h"

class AlgVector;
class VglCamera;

class VGL_API VglViewSurface
{
  VglCamera* m_camera;
public:
  /**
   * Constructs a new VglViewSurface object.
   * \param camera Camera associated with this view surface.
   */
  VglViewSurface (VglCamera* camera)
    : m_camera(camera)
  {
  }
  /**
   * Destroys a VglViewSurface object.
   */
  virtual ~VglViewSurface ()
  {
  }
  /**
   * Centers the canvas cursor.
   */
  void CenterCursor()
  {
    SetCursorPos(0.5f,0.5f);
  }
  /**
   * Returns the camera associated with this view surface.
   */
  virtual VglCamera* GetCamera ()
  {
    return m_camera;
  }
  /**
   * Sets this view surface's camera.
   */
  virtual void SetCamera (VglCamera* camera)
  {
    m_camera = camera;
  }
  /**
   * Performs a redraw operation.
   */
  virtual void Redraw () = 0;
  /**
   * Requests the redraw of the surface when the process becomes idle.
   */
  virtual void PostRedraw () = 0;
  /**
   * Performs the drawing of an item (addressing issues like camera setup, viewport, stereoscopy, etc).
   */
  virtual void DrawItem (VglFunction func, void* data=0) = 0;
  /**
   * Sets the canvas cursor position.
   */
  virtual void SetCursorPos (float x, float y) = 0;
  /**
   * Sets if canvas must draw a raster cursor or not.
   */
  virtual void SetRasterCursor (bool f) = 0;
};

#endif
